const USER_AGENT = CommonFunc.USER_AGENT;

const RESULT_OK = 0;
const RESULT_PARSE_ERROR = 1;
const RESULT_NOT_RSS = 2;
const RESULT_NOT_FOUND = 3;
const RESULT_NOT_AVAILABLE = 4;
const RESULT_ERROR_FAILURE = 5;

var resultStrArray = null;

// XUL Object
var strRes, bmStrRes; // stringbundle Object
var infoRes;          // Added by Higmmer
var bookmarksTree;
var rssItemListBox;
var rssStatusImage;
var rssStatusLabel;
var rssTitleLabel;
var rssItemToolTip;
var rssItemListBoxContextMenu; // Added by Higmmer

var currentFeed = null;
var httpReq;
var prefObserverSageFolder;
var responseXML;
var lastResource;
var rssLoading = false;
var sageFolderID = "";
var enableTooltip = true;
var popupTimeoutId = 0;

//----- Added by Higmmer : Start (inspired by Sage-Too) ----->
var autoCheckFeedsIntervalId = -1;
var prefObserver = {
	showUpdateCount: null,
	hideNoUpdatedFeeds: null,
	autoCheckFeedsEnable: null,
	autoCheckFeedsInterval: null
};
//<----- Added by Higmmer : End -----


function init() {
	checkMigration(); // Added by Higmmer

	bookmarksTree = document.getElementById("bookmarksTree");
	rssItemListBox = document.getElementById("rssItemListBox");
	rssStatusImage = document.getElementById("rssStatusImage");
	rssStatusLabel = document.getElementById("rssStatusLabel");
	rssTitleLabel = document.getElementById("rssTitleLabel");
	rssItemToolTip = document.getElementById("rssItemToolTip");
	rssItemListBoxContextMenu = document.getElementById("rssItemListBoxContextMenu"); // Added by Higmmer

	strRes = document.getElementById("strRes");
	bmStrRes = document.getElementById("bmStrRes");
	infoRes = document.getElementById("infoRes"); // Added by Higmmer
	resultStrArray = new Array(
		strRes.getString("RESULT_OK_STR"),
		strRes.getString("RESULT_PARSE_ERROR_STR"),
		strRes.getString("RESULT_NOT_RSS_STR"),
		strRes.getString("RESULT_NOT_FOUND_STR"),
		strRes.getString("RESULT_NOT_AVAILABLE_STR"),
		strRes.getString("RESULT_ERROR_FAILURE_STR")
	);

	// get feed folder location
	sageFolderID = _impl.getRootFolderId(); // Modified by Higmmer
	// check for changes to the feed folder
	prefObserverSageFolder = CommonFunc.addPrefListener(CommonFunc.FEED_FOLDER_ID, sageFolderChanged);
	// set feed folder location
	_impl.setBookmarksTreePlace(bookmarksTree, sageFolderID); // Modified by Higmmer
	// select first entry
	bookmarksTree.treeBoxObject.view.selection.select(0);

	try { // Added by Higmmer as a workaround for failing initializing issue
		FeedSearch.init();
	} catch(e) {
		logMessage("Fail to initialize search engines");
	}
	toggleShowSearchBar();
	toggleShowFeedItemList();
	toggleShowFeedItemListToolbar();

	//----- Added by Higmmer : Start ----->
	setHideNoUpdated(); // (inspired by Sage-Too)
	CommonFunc.setPrefValue(CommonFunc.HTML_FILTER_METHOD, "int", 1);
	//<----- Added by Higmmer : End -----

	document.documentElement.controllers.appendController(readStateController);
	readStateController.onCommandUpdate();

	//----- Added by Higmmer : Start (inspired by Sage-Too) ----->
	prefObserver.showUpdateCount = CommonFunc.addPrefListener(CommonFunc.SHOW_UPDATE_COUNT, sageShowUpdateCountChanged);
	prefObserver.hideNoUpdatedFeeds = CommonFunc.addPrefListener(CommonFunc.HIDE_NO_UPDATED_FEEDS, sageHideNoUpdatedFeedsChanged);
	prefObserver.autoCheckFeedsEnable = CommonFunc.addPrefListener(CommonFunc.AUTO_CHECK_FEEDS_ENABLE, sageAutoCheckFeedsChanged);
	prefObserver.autoCheckFeedsInterval = CommonFunc.addPrefListener(CommonFunc.AUTO_CHECK_FEEDS_INTERVAL, sageAutoCheckFeedsChanged);

	UpdateCounter.update(null, true, true);
	setupAutoCheckFeedsTimer();
	//<----- Added by Higmmer : End -----

	logMessage("initialized");

	// Added by Higmmer (taken from "Firefox Hacks Translation Diary")
	if(checkAutoCheckFeedsStartup()) window.setTimeout(function(){
		updateCheck(CommonFunc.getPrefValue(CommonFunc.AUTO_CHECK_STARTUP_TARGET, "str", sageFolderID), "startup");
	}, 250);
}

function discoverFeeds() {
	window.openDialog("chrome://sagepp/content/dialogs/discoverFeeds/discoverFeeds.xul", "sage_discover_feeds", "chrome,modal,centerscreen", bookmarksTree); // Modified by Higmmer
}

// TODO: This does not work in 0.9.x since the implementation for smart bookmarks
//       has been removed. Too bad because this feature was really nice
function showOnlyUpdated() { // not used
	if(getCheckboxCheck("chkOnlyUpdate")) {
		var findURL = "find:datasource=rdf:bookmarks&match=";
			findURL += CommonFunc.BM_DESCRIPTION;
			findURL += "&method=is&text=updated";
		bookmarksTree.tree.setAttribute("ref", findURL);
	} else {
		bookmarksTree.tree.setAttribute("ref", sageFolderID);
	}
}

function sageFolderChanged(subject, topic, prefName) { // Modified by Higmmer
	sageFolderID = _impl.changeRootFolderId(_impl.getRootFolderId(), sageFolderID);
	_impl.setBookmarksTreePlace(bookmarksTree, sageFolderID);
	if(bookmarksTree.view.rowCount > 0) bookmarksTree.view.selection.select(0);
}

//----- Added by Higmmer : Start (inspired by Sage-Too) ----->
function sageShowUpdateCountChanged() {
	try{ UpdateCounter.update(null, true, true); } catch(e){}
}

function sageHideNoUpdatedFeedsChanged() {
	setHideNoUpdated();
	_impl.invalidateAllTreeView(bookmarksTree.view);
	if(bookmarksTree.view.rowCount > 0) {
		bookmarksTree.view.selection.select(0);
		bookmarksTree.treeBoxObject.ensureRowIsVisible(0);
	}
}

function sageAutoCheckFeedsChanged(subject, topic, prefName) {
	setupAutoCheckFeedsTimer();
}

function setupAutoCheckFeedsTimer() {
	if(autoCheckFeedsIntervalId >= 0) {
		clearInterval(autoCheckFeedsIntervalId);
		autoCheckFeedsIntervalId = -1;
	}

	if(CommonFunc.getPrefValue(CommonFunc.AUTO_CHECK_FEEDS_ENABLE, "bool", false)) {
		var interval = CommonFunc.getPrefValue(CommonFunc.AUTO_CHECK_FEEDS_INTERVAL, "int", 60);
		if(interval < 5) interval = 5;
		autoCheckFeedsIntervalId = setInterval(autoCheckFeedsIntervalTick, interval * 60 * 1000);
	}
}

function autoCheckFeedsIntervalTick() {
	logMessage("Automatically checking feeds");
	updateCheck(CommonFunc.getPrefValue(CommonFunc.AUTO_CHECK_TARGET, "str", sageFolderID), "timer");
}

function checkAutoCheckFeedsStartup() {
	switch(CommonFunc.getPrefValue(CommonFunc.AUTO_CHECK_FEEDS_STARTUP, "str", "")) {
		case "reserved":
			CommonFunc.setPrefValue(CommonFunc.AUTO_CHECK_FEEDS_STARTUP, "str", "on");
			return true;
		case "on":
			return !CommonFunc.getPrefValue(CommonFunc.AUTO_CHECK_STARTUP_ONCE, "bool", true)
		default:
			return false;
	}
}

function clearFeedsCache() { // Ported from Sage-Too
	if(!CommonFunc.getPrefValue(CommonFunc.RENDER_FEEDS_TO_CACHE, "bool", false)) return;

	var tmpFile = CreateHTML.getOutputDir();
	if(!tmpFile.exists() || !tmpFile.isDirectory()) return;

	logMessage("Clearing feeds cache...");
	var nowTime = (new Date()).getTime();
	var expireTime = CommonFunc.getPrefValue(CommonFunc.CACHE_EXPIRE_DAYS, "int", 14) * 24 * 60 * 60 * 1000;

	var dirEntries = tmpFile.directoryEntries;
	while(dirEntries.hasMoreElements()) {
		try {
			var file = dirEntries.getNext().QueryInterface(Components.interfaces.nsILocalFile);
			if(file.leafName.match(/^[0-9a-f]{40}\.(sagepp\.)?html$/i) && (nowTime - file.lastModifiedTime) > expireTime) {
				file.remove(false);
			}
		} catch(e) {}
	}
}
//<----- Added by Higmmer : End -----

function done() {
	if(prefObserverSageFolder) {
		CommonFunc.removePrefListener(prefObserverSageFolder);
	}

	//----- Added by Higmmer : Start (inspired by Sage-Too) ----->
	for each(var observer in prefObserver) {
		if(observer) CommonFunc.removePrefListener(observer);
	}
	if(autoCheckFeedsIntervalId >= 0) clearInterval(autoCheckFeedsIntervalId);
	//<----- Added by Higmmer : End -----

	if(rssLoading) {
		httpReq.abort();
		rssLoading = false;
	}
	UpdateChecker.done();

	//----- Added by Higmmer : Start ----->
	if(window.top.XULBrowserWindow) window.top.XULBrowserWindow.setOverLink("", null);

	if(!CommonFunc.getPrefValue(CommonFunc.MARK_AS_READ_ON_CLOSING, "bool", false)) currentFeed = null;
	markAllAsReadForFeed(currentFeed, true);

	clearFeedsCache(); // Ported from Sage-Too
	//<----- Added by Higmmer : End -----

	logMessage("shutdown");
}

function openOPMLWizard() {
	var dialogURL = "chrome://sagepp/content/dialogs/opml/opml.xul";
	window.openDialog(dialogURL, "", "chrome,modal,centerscreen"); // Modified by Higmmer
}

function openCacheViewer() { // Added by Higmmer
	var tmpFile = CreateHTML.getOutputDir();
	if(!tmpFile.exists() || !tmpFile.isDirectory()) return;

	var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
	var url = ios.newFileURI(tmpFile).spec;
	getContentBrowser().loadOneTab(url, null, null, null, false, false);
}

function openSettingDialog() { // Modified by Higmmer
	var windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
	var settingDialog = windowManager.getMostRecentWindow("Sagepp:Settings");
	if(settingDialog) {
		settingDialog.focus();
		return;
	}

	var dialogURL = "chrome://sagepp/content/dialogs/settings/settings.xul";
	var instantApply = false;
	try {
		var pref = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces.nsIPrefBranch);
		instantApply = pref.getBoolPref("browser.preferences.instantApply");
	} catch(e) {
	}
	window.openDialog(dialogURL, "", "chrome,toolbar,centerscreen," + (instantApply ? "dialog=no" : "modal"));
}

function openSageProjectFeed() {
	lastResource = null;

	//----- Replaced by Higmmer : Start ----->
	//var feedURL = "http://sage.mozdev.org/rss.xml";
	//setStatusLoading("Sage Project News");
	var feedURL = getInformation(CommonFunc.INFORMATION_FEED_URL, "str") || "http://sagerss.com/feed/";
	var feedTitle = getInformation(CommonFunc.INFORMATION_TITLE, "wstr") || "Sage Project News";
	CreateHTML.tabbed = CommonFunc.getPrefValue(CommonFunc.OPEN_NEW_TAB, "bool", false);
	CreateHTML.forcenewtab = CommonFunc.getPrefValue(CommonFunc.OPEN_NEW_TAB_FOREGROUND, "bool", false);
	setStatusLoading(feedTitle);
	//<----- Replaced by Higmmer : End -----

	httpGet(feedURL);
}

function manageRSSList() { // Modified by Higmmer (inspired by Sage 1.4)
	var query = CommonFunc.getItemAnnotation(sageFolderID, CommonFunc.ANNO_ORG_QUERY, "AllBookmarks");
	var windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
	var organizer = windowManager.getMostRecentWindow("Places:Organizer");
	if(!organizer) {
		window.openDialog("chrome://browser/content/places/places.xul", "", "chrome,toolbar,dialog=no,resizable", query);
	} else {
		organizer.PlacesOrganizer.selectLeftPaneQuery(query);
		organizer.focus();
	}
}

function updateCheck(aCheckFolderId, from) { // Modified by Higmmer
	//----- Added by Higmmer : Start (inspired by Sage-Too) ----->
	var btn = document.getElementById("checkFeeds");
	if(from == "button" && btn.checked) {
		UpdateChecker.done();
		logMessage("checking is aborted");
		auto_update_mode = false;
		UpdateChecker.onChecked(0);
		return;
	} else if(UpdateChecker.checking || auto_update_mode) return;
	btn.checked = true;
	var checkingFolderId = aCheckFolderId || sageFolderID;
	//<----- Added by Higmmer : End -----

	UpdateChecker.onCheck = function(aName, aURL) {
		rssStatusImage.setAttribute("loading", "true");
		rssStatusLabel.value = strRes.getString("RESULT_CHECKING") + ": " + aName;
	}
	UpdateChecker.onChecked = function(aFoundFeeds) { // Modified by Higmmer
		setStatusDone();
		btn.checked = false;
		UpdateCounter.update(null, true);
		doneUpdateCheck(checkingFolderId, aFoundFeeds);
	}

	//----- Added by Higmmer : Start ----->
	// (from Sage-Too)
	checkAutoCheckFeedsStartup();
	setupAutoCheckFeedsTimer();
	// (Sage++ original)
	var enableAutoUpdate, skipChecking;
	switch(from) {
	case "startup":
	case "timer":
		enableAutoUpdate = false;
		break;
	case "openfeeds":
		enableAutoUpdate = skipChecking = true;
		break;
	}
	autoUpdateCleanUp();
	autoUpdateInit(enableAutoUpdate);
	//<----- Added by Higmmer : End -----

	UpdateChecker.startCheck(checkingFolderId, skipChecking); // Modified by Higmmer

	if(render_feeds && auto_update_mode) getContentBrowser().contentWindow.focus(); // Added by Higmmer
}

function showAlertOnUpdated(aFoundFeeds) { // Added by Higmmer
	if(aFoundFeeds == 0 || !CommonFunc.getPrefValue(CommonFunc.ALERT_ON_UPDATE, "bool", true)) return;
	try {
		var alertsService = Components.classes["@mozilla.org/alerts-service;1"].getService(Components.interfaces.nsIAlertsService);
		alertsService.showAlertNotification("chrome://sagepp/skin/sage-icon-32.png",
		  strRes.getString("WINDOW_TITLE"),
		  strRes.getString("ALERT_ONUPDATE_FOUND") + " " + aFoundFeeds + " " + strRes.getString("ALERT_ONUPDATE_FEEDS"),
		  false, "", null, ""
		);
	} catch(e) {}
}

function BookmarkResource(aRes, aDB) { // used on Fx2 only
	this.res = aRes;
	this.db = aDB;
	this.name = BookmarksUtils.getProperty(this.res, CommonFunc.BM_NAME, this.db);
	if(BookmarksUtils.getProperty(this.res, CommonFunc.RDF_TYPE, this.db) == CommonFunc.RDF_TYPE_BOOKMARK) {
		this.url = BookmarksUtils.getProperty(this.res, CommonFunc.BM_URL, this.db);
	}
	if(BookmarksUtils.getProperty(this.res, CommonFunc.RDF_TYPE, this.db) == CommonFunc.RDF_TYPE_LIVEMARK) {
		this.url = BookmarksUtils.getProperty(this.res, CommonFunc.BM_FEEDURL, this.db);
	}
}

function bookmarksOpen(node) { // Modified by Higmmer
	lastResource = node;
	if(_impl.nodeIsLivemarkContainer(node.parent)) {
		if(CreateHTML._tabbed) {
			var newtab = getContentBrowser().addTab(node.uri);
			if(CreateHTML._forcenewtab) getContentBrowser().selectedTab = newtab;
		} else {
			getContentBrowser().loadURI(node.uri);
		}
	} else {
		setStatusLoading();
		httpGet(_impl.getUrl(node));
	}
}

function createTreeContextMenu2(aEvent) { // used on Fx2 only
	var popup = aEvent.target;
	if(popup.localName != "menupopup") return;

	var selection = bookmarksTree._selection;
	var itemId = selection.item[0].Value;
	var cmdSrc = "";
	var tempMenuItem;

	// get type of parent node
	var predicate = bookmarksTree.db.ArcLabelsIn(bookmarksTree.currentResource).getNext();
	if(predicate instanceof Components.interfaces.nsIRDFResource) {
		var parent = bookmarksTree.db.GetSource(predicate, bookmarksTree.currentResource, true);
	}
	var parentType = BookmarksUtils.getProperty(parent, CommonFunc.RDF_TYPE, bookmarksTree.db);

	if((selection.type == "Bookmark" && parentType != CommonFunc.RDF_TYPE_LIVEMARK) || selection.type == "Livemark") {
		cmdSrc = "GetRssTitle.getRssTitle('" + itemId + "')";
		tempMenuItem = document.createElement("menuitem");
		tempMenuItem.setAttribute("label", strRes.getString("GET_RSS_TITLE"));
		tempMenuItem.setAttribute("oncommand", cmdSrc);
		popup.appendChild(document.createElement("menuseparator"));
		popup.appendChild(tempMenuItem);
	} else if(selection.type == "Folder") {
		cmdSrc = "updateCheck('" + itemId + "')";
		tempMenuItem = document.createElement("menuitem");
		tempMenuItem.setAttribute("label", strRes.getString("CHECK_UPDATE"));
		tempMenuItem.setAttribute("oncommand", cmdSrc);
		popup.appendChild(document.createElement("menuseparator"));
		popup.appendChild(tempMenuItem);
	}
}

var LBMLastIndex = -1;
function bookmarksTreeKeyDown(aEvent) { // Added by Higmmer
	LBMLastIndex = -1;
	var view = bookmarksTree.view;
	var index = view.selection.currentIndex; if(index == -1) return;

	if(!_impl.nodeIsLivemarkContainer(view.nodeForTreeIndex(index))) {
		if(aEvent.keyCode == KeyEvent.DOM_VK_RETURN || aEvent.keyCode == KeyEvent.DOM_VK_RIGHT) {
			bookmarksTreeClick(aEvent);
			return;
		}
	} else {
		var btView = bookmarksTree.treeBoxObject.view;
		switch(aEvent.keyCode) {
		case KeyEvent.DOM_VK_LEFT:
			if(btView.isContainerOpen(index)) { btView.toggleOpenState(index); LBMLastIndex = index; }
			return;
		case KeyEvent.DOM_VK_RIGHT:
			if(!btView.isContainerOpen(index)) { btView.toggleOpenState(index); LBMLastIndex = index; }
			if(FirefoxVersion.number >= 13000) bookmarksTreeClick(aEvent);
			return;
		case KeyEvent.DOM_VK_RETURN:
			btView.toggleOpenState(index); bookmarksTreeClick(aEvent);
			return;
		}
	}
	if(!aEvent.altKey) return;

	var key = String.fromCharCode(aEvent.keyCode).toUpperCase();
	if(key == "X") { if(index < view.rowCount - 1) index++; }
	else if(key == "D") { if(index > 0) index--; }
	else return;

	view.selection.select(index);
	bookmarksTree.treeBoxObject.ensureRowIsVisible(index);
	aEvent.preventDefault();
}

function bookmarksTreeKeyUp(aEvent) { // Added by Higmmer
	var LBMLastIndex2 = LBMLastIndex; LBMLastIndex = -1;
	var view = bookmarksTree.view;
	var index = view.selection.currentIndex; if(index == -1) return;
	var selection = view.nodeForTreeIndex(index);

	// Workaround for issue on open/close livemark by arrow-keys
	if(LBMLastIndex2 >= 0 && LBMLastIndex2 != index) {
		view.selection.select(LBMLastIndex2);
		bookmarksTree.treeBoxObject.ensureRowIsVisible(LBMLastIndex2);
		aEvent.preventDefault();
		return;
	}

	if(aEvent.keyCode == KeyEvent.DOM_VK_SPACE) {
		if(PlacesUtils.nodeIsFolder(selection) && !_impl.nodeIsLivemarkContainer(selection)) {
			bookmarksTree.treeBoxObject.view.toggleOpenState(index);
		}
		else bookmarksTreeClick(aEvent);
		return;
	}
	if(!aEvent.altKey) return;

	var key = String.fromCharCode(aEvent.keyCode).toUpperCase();
	switch(key) {
	case "A":
		var checkOpt = "openfeeds";
		// fall through
	case "C":
		if(PlacesUtils.nodeIsFolder(selection) && !_impl.nodeIsLivemarkContainer(selection))
			updateCheck(view.nodeForTreeIndex(index).itemId, checkOpt);
		break;
	default:
		return;
	}

	aEvent.preventDefault();
}

function bookmarksTreeClick(aEvent) {
	//----- Modified by Higmmer : Start (inspired by Sage-Too) ----->
	var node = bookmarksTree.controller._view.selectedNode;
	if(!node) return;
	var selectedItemType = node.type;
	//<----- Modified by Higmmer : End -----

	switch(aEvent.type) {
		case "click":
			var obj = {};
			var row = {};
			bookmarksTree.treeBoxObject.getCellAt(aEvent.clientX, aEvent.clientY, row, {}, obj);
			row = row.value;
			if(aEvent.button == 2 || aEvent.originalTarget.localName != "treechildren" || row == -1) {
				return;
			}
			if(obj.value == "twisty") return;

			//----- Modified by Higmmer : Start ----->
			if(PlacesUtils.nodeIsFolder(node) && !_impl.nodeIsLivemarkContainer(node)) {
				if(aEvent.button == 1 || aEvent.ctrlKey) updateCheck(node.itemId);
				else if(aEvent.shiftKey) updateCheck(node.itemId, "openfeeds");
				else bookmarksTree.treeBoxObject.view.toggleOpenState(row);
				return;
			}
			break;
			//<----- Modified by Higmmer : End -----

		case "keypress":
			if(aEvent.originalTarget.localName != "tree") {
				return;
			}
			break;
	}

	//----- Modified by Higmmer : Start ----->
	// (inspired by Sage-Too)
	if(selectedItemType == node.RESULT_TYPE_SEPARATOR
	|| (selectedItemType == node.RESULT_TYPE_FOLDER && !_impl.nodeIsLivemarkContainer(node))
	|| selectedItemType == node.RESULT_TYPE_FOLDER_SHORTCUT) {
		return;
	}

	var openNewTab = CommonFunc.getPrefValue(CommonFunc.OPEN_NEW_TAB, "bool", false);
	var openInForeground = CommonFunc.getPrefValue(CommonFunc.OPEN_NEW_TAB_FOREGROUND, "bool", false);
	if(aEvent.button == 1 && aEvent.shiftKey) { // shift + middle click
		CreateHTML.tabbed = true;
		CreateHTML.forcenewtab = true;
	} else if(aEvent.button == 1 || aEvent.ctrlKey) { // click middle button or press ctrl Key
		CreateHTML.tabbed = true;
		CreateHTML.forcenewtab = openNewTab ^ openInForeground;
	} else {
		CreateHTML.tabbed = openNewTab;
		CreateHTML.forcenewtab = openInForeground;
	}
	//<----- Modified by Higmmer : End -----

	bookmarksOpen(node);
}

function rssItemListBoxMouseMove(aEvent) { // Added by Higmmer
	var target = aEvent.originalTarget;
	if(target.localName != "listitem") {
		window.top.XULBrowserWindow.setOverLink("", null);
		return;
	}
	var feedItem = getFeedItemFromListItem(target);
	window.top.XULBrowserWindow.setOverLink(feedItem.getLink(), null);
}

function rssItemListBoxKeyUp(aEvent) { // Added by Higmmer
	if(aEvent.keyCode == KeyEvent.DOM_VK_SPACE) {
		rssItemListBoxClick(aEvent);
		return;
	}
}

function rssItemListBoxClick(aEvent) {
	if(aEvent.type == "click") {
		if(aEvent.button == 2 || aEvent.originalTarget.localName != "listitem") {
			return;
		}
	} else if(aEvent.type == "keypress") {
		if(aEvent.originalTarget.localName != "listbox") {
			return;
		}
	}

	var listItem = rssItemListBox.selectedItem;
	var feedItem = getFeedItemFromListItem( listItem );

	listItem.setAttribute("visited", "true");
	openURI(feedItem.getLink(), aEvent);
}

function rssTitleLabelClick(aNode, aEvent) {
	var tabbed = false;
	if(!currentFeed || aEvent.button == 2) {
		return;
	}

	var link = currentFeed.getLink();
	openURI(link, aEvent);
}

function setStatusLoading(label) {
	rssStatusImage.setAttribute("loading", "true");
	if(label) {
		rssStatusLabel.value = strRes.getString("RESULT_LOADING") + ": " + label;
	} else {
		// Modified by Higmmer (inspired by Sage-Too)
		rssStatusLabel.value = strRes.getString("RESULT_LOADING") + ": " + lastResource.title;
	}
}

function setStatusDone() {
	rssStatusImage.setAttribute("loading", "false");
	rssStatusLabel.value = "";

	if(currentFeed) {
		rssTitleLabel.value = currentFeed.getDecodedTitle(); // Modified by Higmmer
		if(currentFeed.getLink()) {
			rssTitleLabel.setAttribute("class", "link");
			rssTitleLabel.tooltipText = currentFeed.getLink();
		} else {
			rssTitleLabel.removeAttribute("class");
			rssTitleLabel.tooltipText = "";
		}
	}
}

function setStatusError(aStatus) {
	rssStatusImage.setAttribute("loading", "error");
	rssStatusLabel.value = "Error: " + aStatus;
}

function getContentBrowser() {
	var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
	var topWindowOfType = windowManager.getMostRecentWindow("navigator:browser");
	if (topWindowOfType) {
		return topWindowOfType.document.getElementById('content');
	}
	return null;
}

function isCurrentTabBlank(browser) { // Added by Higmmer
	if(!browser) browser = getContentBrowser();
	var selectedBrowser = browser.selectedBrowser;
	return selectedBrowser.currentURI.spec == "about:blank" && !selectedBrowser.webProgress.isLoadingDocument
}

function toggleShowSearchBar() {
	var showSearchBar = getCheckboxCheck("chkShowSearchBar");
	document.getElementById("searchseparator").hidden = !showSearchBar; // Added by Higmmer (from Sage-Too)
	document.getElementById("barSearch").hidden = !showSearchBar;
}

function toggleShowFeedItemList() {
	var showFeedItemList = getCheckboxCheck("chkShowFeedItemList");
	document.getElementById("sage-splitter").hidden = !showFeedItemList;
	document.getElementById("rssItemListBoxBox").hidden = !showFeedItemList;
	if(showFeedItemList) setRssItemListBox();
}

function toggleShowFeedItemListToolbar() {
	var showFeedItemListToolbar = getCheckboxCheck("chkShowFeedItemListToolbar");
	document.getElementById("listboxseparator").hidden = !showFeedItemListToolbar; // Added by Higmmer
	document.getElementById("itemListToolbar").hidden = !showFeedItemListToolbar;
	if(showFeedItemListToolbar) readStateController.onCommandUpdate();
}

//----- Added by Higmmer : Start ----->
function setHideNoUpdated() { // (inspired by Sage-Too)
	document.getElementById("hideNoUpdated").checked = CommonFunc.getPrefValue(CommonFunc.HIDE_NO_UPDATED_FEEDS, "bool", false);
}

function toggleHideNoUpdated(button) { // (inspired by Sage-Too)
	CommonFunc.setPrefValue(CommonFunc.HIDE_NO_UPDATED_FEEDS, "bool", !button.checked);
}

function toggleAllFolders() { // (from Sage-Too)
	var view = bookmarksTree.view;
	var allopen = true;
	for(var i = 0; i < view.rowCount; i++) {
		if(view.isContainer(i)) {
			if(FirefoxVersion.number >= 13000 && _impl.nodeIsLivemarkContainer(view.nodeForTreeIndex(i))) {
				continue;
			}
			allopen = allopen && view.isContainerOpen(i);
		}
	}
	allopen = !allopen;
	var change, lastIndex = Infinity;
	do {
		change = false;
		for(var i = view.rowCount - 1; i >= 0; i--) {
			if(!view.isContainer(i)) continue;
			if(FirefoxVersion.number >= 13000 && _impl.nodeIsLivemarkContainer(view.nodeForTreeIndex(i))) {
				if(view.isContainerOpen(i)) view.toggleOpenState(i);
				continue;
			}
			if(view.isContainerOpen(i) != allopen) {
				view.toggleOpenState(i);
				change = true;
				if(allopen && i < lastIndex) {
					view.selection.select(i);
					bookmarksTree.treeBoxObject.ensureRowIsVisible(i);
					lastIndex = i;
				}
			}
		}
	} while(allopen && change);
	if(view.rowCount > 0 && (!allopen || view.selection.currentIndex < 0)) {
		view.selection.select(0);
		bookmarksTree.treeBoxObject.ensureRowIsVisible(0);
	}
}

function toggleHGrenderFeeds() {
	CommonFunc.setPrefValue(CommonFunc.RENDER_FEEDS, "bool", getCheckboxCheck("chkHGrenderFeeds"));
}

function toggleHGCreateHTMLTabbed() {
	CommonFunc.setPrefValue(CommonFunc.OPEN_NEW_TAB, "bool", getCheckboxCheck("chkHGCreateHTMLTabbed"));
}

function toggleHGOpenNewTabForeground() {
	CommonFunc.setPrefValue(CommonFunc.OPEN_NEW_TAB_FOREGROUND, "bool", getCheckboxCheck("chkHGOpenNewTabForeground"));
}

function toggleHGEnableAutoUpdate() {
	CommonFunc.setPrefValue(CommonFunc.AUTO_UPDATE_ENABLE, "bool", getCheckboxCheck("chkHGEnableAutoUpdate"));
}

function toolmenuPopupShowing() {
	[
		["chkHGrenderFeeds", CommonFunc.RENDER_FEEDS, "bool", true],
		["chkHGCreateHTMLTabbed", CommonFunc.OPEN_NEW_TAB, "bool", false],
		["chkHGOpenNewTabForeground", CommonFunc.OPEN_NEW_TAB_FOREGROUND, "bool", false],
		["chkHGEnableAutoUpdate", CommonFunc.AUTO_UPDATE_ENABLE, "bool", false],
	].forEach(function(item){
		var elem, pref, type, defValue;
		[elem, pref, type, defValue] = item;
		setCheckbox(elem, CommonFunc.getPrefValue(pref, type, defValue));
	});

	var infoTitle = getInformation(CommonFunc.INFORMATION_TITLE, "wstr");
	if(!infoTitle) return;
	document.getElementById("menuInformation").label = infoTitle;
}

function getInformation(name, type) {
	var info = "";
	try {
		info = CommonFunc.prefHasUserValue(name) ? CommonFunc.getPrefValue(name, type, "") : infoRes.getString(CommonFunc.PREFS_PREFIX + name);
	} catch(e) {
	}
	return info;
}
//<----- Added by Higmmer : End -----

function setRssItemListBox() {
	if(!currentFeed) return;
	if(document.getElementById("rssItemListBoxBox").hidden) return;

	while(rssItemListBox.getRowCount() != 0) {
		rssItemListBox.removeItemAt(0);
	}

	var feedItemOrder = CommonFunc.getPrefValue(CommonFunc.FEED_ITEM_ORDER, "str", "chrono");

	var items = currentFeed.getItems(feedItemOrder);

	for(var i = 0; items.length > i; i++) {
		var item = items[i];
		var itemLabel = item.getDecodedTitle(); // Modified by Higmmer
		itemLabel = (i+1) + ". " + itemLabel;
		var listItem = rssItemListBox.appendItem(itemLabel, i);
		if(linkVisitor.getVisited(item.getLink())) {
			listItem.setAttribute("visited", "true");
		}
	}

	readStateController.onCommandUpdate();
}

function getCheckboxCheck(element_id) {
	var checkboxNode = document.getElementById(element_id);
	return checkboxNode.getAttribute("checked") == "true";
}

function setCheckbox(element_id, value) {
	var checkboxNode = document.getElementById(element_id);
	checkboxNode.setAttribute("checked", value);
}


function populateToolTip(e) {
	// if setting disabled
	if(!getCheckboxCheck("chkShowTooltip")) {
		e.preventDefault();
		return;
	}

	if(document.tooltipNode == rssItemListBox) {
		e.preventDefault();
		return;
	}

	//----- Added by Higmmer : Start ----->
	switch(rssItemListBoxContextMenu.state){
	case "open":
	case "showing":
		e.preventDefault();
		return;
	}
	//<----- Added by Higmmer : End -----

	var listItem = document.tooltipNode;
	if(listItem.value === undefined){ // Added by Higmmer
		e.preventDefault();
		return;
	}

	var feedItemOrder = CommonFunc.getPrefValue(CommonFunc.FEED_ITEM_ORDER, "str", "chrono");
	var items = currentFeed.getItems(feedItemOrder);
	var description = entityDecode(htmlToText(items[listItem.value].getContent())); // Modified by Higmmer
	if(description.indexOf("/") != -1) {
		description = description.replace(/\//gm, "/\u200B");
	}
	if(description.length > 400) {
		description = description.substring(0, 400) + "...";
	}

	rssItemToolTip.title = listItem.label;
	rssItemToolTip.description = description;
}

//----- Removed by Higmmer : Start (moved to commonfunc.js)----->
// Fix: Script Injection Vulnerability
// Some character entities can turn into HTML tags although "Allow HTML tags" is disabled.
// ( Each [M] mark means "This line is modified by Higmmer". )
/*function htmlToText(aStr) {
	var	formatConverter = Components.classes["@mozilla.org/widget/htmlformatconverter;1"].createInstance(Components.interfaces.nsIFormatConverter);
	var fromStr = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString);
	fromStr.data = aStr.replace(/&/g, "&amp;"); // [M]
	var toStr = { value: null };

	try {
		formatConverter.convert("text/html", fromStr, fromStr.toString().length, "text/unicode", toStr, {});
	} catch(e) {
		return "Converting Error"; // [M]
	}
	if(toStr.value) {
		toStr = toStr.value.QueryInterface(Components.interfaces.nsISupportsString);
		return toStr.toString().replace(/&(?!([a-z]+|#[0-9]+|#x[0-9a-f]+);)/gi, "&amp;"); // [M]
	}
	return "Converting Error"; // [M]
}*/
//<----- Removed by Higmmer : End -----



// ++++++++++ +++++++++  HTTP	++++++++++ +++++++++

function httpGet(aURL, aMode) { // Modified by Higmmer
	if(rssLoading) {
		httpReq.abort();
		rssLoading = false;
	}
	if(!aMode) auto_update_mode = false; // Added by Higmmer

	responseXML = null;

	httpReq = new XMLHttpRequest();

	httpReq.open("GET", aURL);

	httpReq.onload = httpLoaded;
	httpReq.onerror = httpError;
	httpReq.onreadystatechange = httpReadyStateChange;

	try {
		httpReq.setRequestHeader("User-Agent", USER_AGENT);
		httpReq.overrideMimeType("application/xml");
	} catch(e) {
		httpGetResult(RESULT_ERROR_FAILURE);
	}

	try {
		httpReq.send(null);
		rssLoading = true;
	} catch(e) {
		httpGetResult(RESULT_ERROR_FAILURE);
	}
}

function httpError(e) {
	logMessage("HTTP Error: " + e.target.status + " - " + e.target.statusText);
	httpGetResult(RESULT_NOT_AVAILABLE);
}

function httpReadyStateChange() {
	if(httpReq.readyState == 2) {
		try {
			if(httpReq.status == 404) {
				httpGetResult(RESULT_NOT_FOUND);
			}
		} catch(e) {
			httpGetResult(RESULT_NOT_AVAILABLE);
			return;
		}
	} else if(httpReq.readyState == 3) {}
}

function httpLoaded(e) {
	// Added by Higmmer : Start (from Sage-Too) ----->
	var location = httpReq.getResponseHeader("location");
	if(location){ httpGet(location); return; }
	//<----- Added by Higmmer : End -----

	responseXML = httpReq.responseXML;
	var rootNodeName = responseXML.documentElement.localName.toLowerCase();

	if(rootNodeName == "parsererror") { // Added by Higmmer
		responseXML = tryFixInvalidXml(httpReq.responseText);
		if(responseXML != null) rootNodeName = responseXML.documentElement.localName.toLowerCase();
	}

	switch(rootNodeName) {
		case "parsererror":
			// XML Parse Error
			httpGetResult(RESULT_PARSE_ERROR);
			break;
		case "rss":
		case "rdf":
		case "feed":
			httpGetResult(RESULT_OK);
			break;
		default:
			// Not RSS or Atom
			httpGetResult(RESULT_NOT_RSS);
			break;
	}
}

function httpGetResult(aResultCode) {
	httpReq.abort();
	rssLoading = false;

	if(aResultCode == RESULT_OK) {
		var feed = new Feed(responseXML, lastResource ? lastResource.uri : null); // Modified by Higmmer

		//----- Replaced by Higmmer : Start ----->
		/*if(lastResource && lastResource.res) {
			if(CommonFunc.getPrefValue(CommonFunc.AUTO_FEED_TITLE, "bool", false)) {
				if(CommonFunc.getBMDSProperty(lastResource.res, CommonFunc.BM_NAME) != currentFeed.getTitle()) {
					CommonFunc.setBMDSProperty(lastResource.res, CommonFunc.BM_NAME, currentFeed.getTitle());
				}
			}

			BMSVC.updateLastVisitedDate(lastResource.url, "UTF-8");
			CommonFunc.setBMDSProperty(lastResource.res, CommonFunc.BM_DESCRIPTION, CommonFunc.STATUS_NO_UPDATE + " " + currentFeed.getSignature());
		}*/
		updateFeedProperty(feed, lastResource);
		//<----- Replaced by Higmmer : End -----

		if(auto_update_mode) { // Added by Higmmer
			autoUpdateRenderProc(feed);
			return;
		}
		if(CommonFunc.getPrefValue(CommonFunc.MARK_AS_READ_ON_CLOSING, "bool", false)) markAllAsReadForFeed(currentFeed); // Added by Higmmer
		currentFeed = feed;

		setStatusDone();
		setRssItemListBox();

		if(CommonFunc.getPrefValue(CommonFunc.RENDER_FEEDS, "bool", true)) {
			CreateHTML.openHTML(currentFeed);
		}
	} else {
		setStatusError(resultStrArray[aResultCode]);

		//----- Added by Higmmer : Start (inspired by Sage-Too) ----->
		_impl.setStatusAndSignature(lastResource, CommonFunc.STATUS_ERROR, null);
		UpdateCounter.update(UpdateCounter._getParent(lastResource), false);
		//<----- Added by Higmmer : End -----

		if(auto_update_mode) { // Added by Higmmer
			autoUpdateRenderProc(null);
			return;
		}
	}

	auto_update_mode = false; // Added by Higmmer
}

//----- Added by Higmmer : Start ----->
function tryFixInvalidXml(aStr) {
	try {
		// Fix unencoded ampersands and undefined entities
		var str = aStr.replace(/&(?!(amp|apos|gt|lt|quot|#[0-9]+|#x[0-9a-f]+);)/gi, "&amp;");

		// Remove invalid control characters
		str = str.replace(/[\x00-\x08\x0b\x0c\x0e-\x1f]/g, " ");

		var parser = new DOMParser();
		return parser.parseFromString(str, "application/xml");
	} catch(e) {
		return null;
	}
}

// This code fragment is taken from httpGetResult(), plus inspired by Sage-Too
function updateFeedProperty(aFeed, aRes) {
	if(aRes) {
		if(CommonFunc.getPrefValue(CommonFunc.AUTO_FEED_TITLE, "bool", false)) {
			// Modified by Higmmer
			var title = aFeed.getDecodedTitle();
			if(aRes.title != title) {
				var txn = PlacesUIUtils.ptm.editItemTitle(aRes.itemId, title);
				PlacesUIUtils.ptm.doTransaction(txn);
			}
		}

		// Modified by Higmmer
		//CommonFunc.addVisit(aRes); // removed by Higmmer
		_impl.setLastModified(aRes, aFeed.hasLastPubDate() ? aFeed.getLastPubDate().getTime() : 0);
		_impl.setStatusAndSignature(aRes, CommonFunc.STATUS_NO_UPDATE, aFeed.getSignature());
		var forceRefresh = !CommonFunc.getPrefValue(CommonFunc.SHOW_UPDATE_COUNT, "bool", false);
		UpdateCounter.update(UpdateCounter._getParent(aRes), false, forceRefresh);
	}
}
//<----- Added by Higmmer : End -----

// This takes a list item from the rss list box and returns the uri it represents
// this seems a bit inefficient. Shouldn't there be a direct mapping between these?

/**
 * This takes a listitem element and returns the FeedItem it represents
 * @param	oListItem : XULListItem
 * @returns	FeedItem
 */
function getFeedItemFromListItem( oListItem ) {
	var feedItemOrder = CommonFunc.getPrefValue(CommonFunc.FEED_ITEM_ORDER, "str", "chrono");
	var items = currentFeed.getItems(feedItemOrder);
	return items[oListItem.value];
}


/**
 * Opens a link in the same window, a new tab or a new window
 *
 * @param	sURI : String
 * @param	oType : Object	If this is an Event object we check the modifiers.
 * 							Otherwise we assume it is a string describing the
 *                          window type.
 * @returns	void
 */
function openURI(sURI, oType) {
	//----- Modified by Higmmer : Start ----->
	var openNewTab = CommonFunc.getPrefValue(CommonFunc.OPEN_NEW_TAB, "bool", false);
	var openInForeground = CommonFunc.getPrefValue(CommonFunc.OPEN_NEW_TAB_FOREGROUND, "bool", false);
	var windowType = "", forceNewTab = false;
	if (oType instanceof Event) {
		// figure out what kind of open we want
		if (oType.button == 1 && oType.shiftKey) { // shift + middle click
			windowType = "tab";
			forceNewTab = true;
		} else if (oType.button == 1 || oType.ctrlKey) { // click middle button or press ctrl Key
			windowType = "tab";
			forceNewTab = openNewTab ^ openInForeground;
		} else if (oType.shiftKey) {
			windowType = "window";
		} else if (openNewTab) {
			windowType = "tab";
			forceNewTab = openInForeground;
		}
	} else {
		if (openNewTab) {
			windowType = "tab";
		} else {
			windowType = oType;
		}
		forceNewTab = openInForeground;
	}

	var browser = getContentBrowser();
	if (windowType == "tab" && isCurrentTabBlank(browser)) windowType = "";

	switch (windowType ||null) { // Tweaked by Higmmer to prevent TabMixPlus from overwriting
		case "tab":
			var newtab = browser.addTab(sURI);
			if(forceNewTab) browser.selectedTab = newtab;
			break;
		case "window":
			document.commandDispatcher.focusedWindow.open(sURI);
			break;

		default:
			browser.loadURI(sURI);
	}
	//<----- Modified by Higmmer : End -----

	readStateController.onCommandUpdate();
}

/**
 * This is called by the context menu
 * @param	oType : String
 * @returns	void
 */
function openListItem(oType) {
	var listItem = document.popupNode;
	var feedItem = getFeedItemFromListItem(listItem);
	listItem.setAttribute("visited", "true");
	openURI(feedItem.getLink(), oType);
}

//----- Added by Higmmer : Start ----->
function openListItems(bUnreadOnly) {
	if (!currentFeed || currentFeed.length == 0) return false;

	var openURIs = [];
	var feedItems = currentFeed.getItems(CommonFunc.getPrefValue(CommonFunc.FEED_ITEM_ORDER, "str", "chrono"));

	for (var i = 0, feedItem; feedItem = feedItems[i]; i++) {
		var url = feedItem.getLink();
		if (url == "about:blank") continue;

		if (bUnreadOnly) {
			if (!linkVisitor.getVisited(url)) openURIs.push(url);
		} else {
			openURIs.push(url);
		}
	}
	if (openURIs.length == 0 || !PlacesUIUtils._confirmOpenInTabs(openURIs.length, window)) return false;

	var browser = getContentBrowser();
	var loadInBackground = !CommonFunc.getPrefValue(CommonFunc.OPEN_NEW_TAB_FOREGROUND, "bool", false);

	if (isCurrentTabBlank(browser)) {
		browser.loadURI(openURIs.shift());
		loadInBackground = true;
	}
	if (openURIs.length > 0) browser.loadTabs(openURIs, loadInBackground);

	for (var i = 0, item; item = rssItemListBox.getItemAtIndex(i); i++) {
		item.setAttribute("visited", "true");
	}

	return true;
}
//<----- Added by Higmmer : End -----


// link visit code based on LinkVisitor.mozdev.org
var linkVisitor = {
	_uriFixup  : Components.classes["@mozilla.org/docshell/urifixup;1"].getService(Components.interfaces.nsIURIFixup),
	_obsService: Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService), // Added by Higmmer

	//----- Added by Higmmer : Start ----->
	// Fix: Changing visited status for multiple items may cause temporary hang-up on Firefox4
	_visitedItemsQueue : null,
	setVisitedForAllFeedItems : function (aFeedItems, bRead, bWaitForDone) {
		function VisitedStatus (aTitle, aLink, bRead) {
			this.title = aTitle; this.link = aLink; this.read = bRead;
		}

		if (!this._visitedItemsQueue) this._visitedItemsQueue = new Array();
		for (var i = 0, feedItem; feedItem = aFeedItems[i]; i++) {
			this._visitedItemsQueue.push(new VisitedStatus(feedItem.getDecodedTitle(), feedItem.getLink(), bRead));
		}

		if (bWaitForDone) {
			var status;
			while (status = this._visitedItemsQueue.shift()) {
				this.setVisited(status.link, status.read, status.title);
			}
		} else {
			function visitorThreadGenerator() {
				var status;
				while (status = linkVisitor._visitedItemsQueue.shift()) {
					linkVisitor.setVisited(status.link, status.read, status.title);
					yield true;
				}
				yield false;
			}
			var thread = visitorThreadGenerator();
			window.setTimeout(function(){
				if (thread.next()) window.setTimeout(arguments.callee, 0);
				else thread.close();
			}, 0);
		}
	},
	//<----- Added by Higmmer : End -----

	setVisited : function (sURI, bRead, sTitle) { // Modified by Higmmer (from Sage-Too)
		if (!sURI)
			return;

		if (bRead) {
			var IOService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
			var hist = Components.classes["@mozilla.org/browser/global-history;2"];
			//----- Replaced by Higmmer : Start ----->
			//var history = hist.getService(Components.interfaces.nsIGlobalHistory2);
			//history.addURI(IOService.newURI(sURI, null, null), false, true, null);
			var history = hist.getService(Components.interfaces.nsIBrowserHistory);
			history.addPageWithDetails(IOService.newURI(sURI, null, null), "[Sage++] " + (sTitle || sURI), Date.now() * 1000);
			//<----- Replaced by Higmmer : End -----
		} else {
			var IOService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
			var hist = Components.classes["@mozilla.org/browser/global-history;2"];
			var history = hist.getService(Components.interfaces.nsIBrowserHistory);
			history.removePage(IOService.newURI(sURI, null, null));
		}
	},

	getVisited : function (sURI) { // Modified by Higmmer (from Sage-Too)
		var IOService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
		var hist = Components.classes["@mozilla.org/browser/global-history;2"];
		var history = hist.getService(Components.interfaces.nsIGlobalHistory2);
		return history.isVisited(IOService.newURI(sURI, null, null));
	},

	_getFixupURI : function (sURI) {
		try {
			return this._uriFixup.createFixupURI(sURI, 0);
		}
		catch (e) {
			logMessage("Could not fixup URI: " + sURI);
			return null;
		}
	},

	init : function () {
	}
};
linkVisitor.init();


// RSS Item Context Menu

/**
 * This is called before the context menu for the listbox is shown. Here we
 * enabled/disable menu items as well as change the text to correctly reflect
 * the read state
 * @returns	void
 */
function updateItemContextMenu(event) {
	if(rssItemListBox.getRowCount() == 0) { // Added by Higmmer
		event.preventDefault();
		return;
	}

	readStateController.onCommandUpdate(null); // Tweaked by Higmmer to prevent TabMixPlus from overwriting
	document.getElementById("rssMarkAsReadItem").hidden =
		!readStateController.isCommandEnabled("cmd_markasread");
	document.getElementById("rssMarkAsUnreadItem").hidden =
		!readStateController.isCommandEnabled("cmd_markasunread");
}


/**
 * Marks all read or unread
 * @param	bRead : Boolean	Whether to mark as read or unread
 * @returns	void
 */
function markAllReadState(bRead) {
	if (currentFeed) {
		var feedItemOrder = CommonFunc.getPrefValue(CommonFunc.FEED_ITEM_ORDER, "str", "chrono");
		var feedItems = currentFeed.getItems(feedItemOrder);

		//----- Replaced by Higmmer : Start ----->
		//for (var i = 0; i < feedItems.length; i++)
		//	linkVisitor.setVisited(feedItems[i].getLink(), bRead, feedItems[i].getDecodedTitle()); // Modified by Higmmer
		linkVisitor.setVisitedForAllFeedItems(feedItems, bRead, false);
		//<----- Replaced by Higmmer : End -----

		var listItem;
		for (var y = 0; y < rssItemListBox.getRowCount(); y++)
		{
			listItem = rssItemListBox.getItemAtIndex(y);
			if (bRead)
				listItem.setAttribute("visited", "true");
			else
				listItem.removeAttribute("visited");
		}
	}
}


/**
 * This marks the selected items as read/unread. This works with multiple
 * selection as well if we want to enable that in the future.
 * @param	bRead : Boolean		Whether to mark items read or unread
 * @returns	void
 */
function markReadState(bRead) {
	var listItems = rssItemListBox.selectedItems;
	for (var i = 0; i < listItems.length; i++) {
		var listItem = listItems[i];
		var feedItem = getFeedItemFromListItem(listItem);
		var uri = feedItem.getLink();
		var title = feedItem.getDecodedTitle(); // Added by Higmmer
		if (bRead)
			listItem.setAttribute("visited", "true");
		else
			listItem.removeAttribute("visited");
		linkVisitor.setVisited(uri, bRead, title); // Modified by Higmmer
	}
}

/**
 * This toggles the selected items read state. This works with multiple
 * selection as well if we want to enable that in the future.
 *
 * In Thunderbird, pressing M marks all read on unread based on the first
 * item. This seems more consistent and more useful
 *
 * @returns	void
 */
function toggleMarkAsRead() {
	var listItems = rssItemListBox.selectedItems;
	var read;
	for (var i = 0; i < listItems.length; i++) {
		var listItem = listItems[i];
		var feedItem = getFeedItemFromListItem(listItem);
		var uri = feedItem.getLink();
		var title = feedItem.getDecodedTitle(); // Added by Higmmer
		if (read == null)
			read = !linkVisitor.getVisited(uri);
		if (read)
			listItem.setAttribute("visited", "true");
		else
			listItem.removeAttribute("visited");
		linkVisitor.setVisited(uri, read, title); // Modified by Higmmer
	}
}

function markAllAsReadForFeed(aFeed, bWaitForDone) { // Added by Higmmer
	if (aFeed) {
		var feedItemOrder = CommonFunc.getPrefValue(CommonFunc.FEED_ITEM_ORDER, "str", "chrono");
		var feedItems = aFeed.getItems(feedItemOrder);
		linkVisitor.setVisitedForAllFeedItems(feedItems, true, bWaitForDone);
	} else if (bWaitForDone) {
		linkVisitor.setVisitedForAllFeedItems([], true, true);
	}
}


/**
 * This controller object takes care of the commands related to marking feed
 * items as read
 */
var readStateController = {
	supportsCommand : function(cmd) {
		switch (cmd) {
			case "cmd_markasread":
			case "cmd_markasunread":
			case "cmd_toggleread":
			case "cmd_markallasread":
			case "cmd_markallasunread":
			case "cmd_openunreadinnewtab": // Added by Higmmer
				return true;
			default:
				return false;
		}
	},

	isCommandEnabled : function(cmd, stateOverride) { // Modified by Higmmer
		var items, feedItem, visited, i;

		if (!getCheckboxCheck("chkShowFeedItemList"))
			return false;

		switch (cmd) {
			// Enable if any items available. A more exact solution is to loop
			// over the item and disable/enable dependiong on whether all items
			// are read/unread. This solution is however too slow to be practical.
			case "cmd_markallasread":
			case "cmd_markallasunread":
			case "cmd_openunreadinnewtab": // Added by Higmmer
				return rssItemListBox.getRowCount() > 0;

			// There is a state where we mark a listitem as visited even though
			// we don't know if the server will respond and therefore the link
			// might be unread in the history and read in the UI. In these cases
			// both mark as read and mark as unread needs to be enabled

			case "cmd_markasread":
				items = rssItemListBox.selectedItems;

				if(items.length > 0 && stateOverride != null) return !stateOverride; // Added by Higmmer

				// if we have one non visited we can mark as read
				for (i = 0; i < items.length; i++) {
					feedItem = getFeedItemFromListItem( items[i] );
					visited = linkVisitor.getVisited( feedItem.getLink() );
					if (!visited || items[i].getAttribute("visited") != "true")
						return true;
				}
				return false;

			case "cmd_markasunread":
				items = rssItemListBox.selectedItems;

				if(items.length > 0 && stateOverride != null) return !!stateOverride; // Added by Higmmer

				// if we have one visited we can mark as unread
				for (i = 0; i < items.length; i++) {
					feedItem = getFeedItemFromListItem( items[i] );
					visited = linkVisitor.getVisited( feedItem.getLink() );
					if (visited || items[i].getAttribute("visited") == "true")
						return true;
				}
				return false;

			case "cmd_toggleread":
				return this.isCommandEnabled("cmd_markasread") ||
					   this.isCommandEnabled("cmd_markasunread");
		}

		return false;
	},
	doCommand : function(cmd) {
		var stateOverride; // Added by Higmmer

		switch (cmd) {
			case "cmd_markasread":
				markReadState(true);
				break;

			case "cmd_markasunread":
				markReadState(false);
				break;

			case "cmd_toggleread":
				toggleMarkAsRead();
				break;

			case "cmd_markallasread":
				markAllReadState(true);
				stateOverride = true; // Added by Higmmer
				break;

			case "cmd_markallasunread":
				markAllReadState(false);
				stateOverride = false; // Added by Higmmer
				break;

			case "cmd_openunreadinnewtab": // Added by Higmmer
				if(openListItems(true)) stateOverride = true;
				break;
		}
		this.onCommandUpdate(stateOverride); // Modified by Higmmer
	},

	onCommandUpdate: function (stateOverride) { // Modified by Higmmer
		var commands = ["cmd_markasread", "cmd_markasunread",
						"cmd_toggleread",
						"cmd_markallasread", "cmd_markallasunread",
						"cmd_openunreadinnewtab"]; // Modified by Higmmer
		for (var i = 0; i < commands.length; i++)
			goSetCommandEnabled(commands[i], this.isCommandEnabled(commands[i], stateOverride)); // Modified by Higmmer
	},

	onEvent : function(evt) {}
};


//----- Added by Higmmer : Start ----->
function checkMigration() {
	var migrateFrom = CommonFunc.getPrefValue(CommonFunc.NEED_TO_MIGRATE, "str", "");
	if(!migrateFrom) return;
	var migrateTo = CommonFunc.getPrefValue(CommonFunc.LAST_VERSION, "str", "");

	try {
		if(!migrateTo) throw "Current version is unspecified";
		logMessage("Migrating from Version " + migrateFrom + " to " + migrateTo);
		migrate(migrateFrom, migrateTo);
		CommonFunc.clearPref(CommonFunc.NEED_TO_MIGRATE);
		logMessage("Migration completed");
	} catch(e) {
		var msg = "Migration error:\n" + e.toString();
		logMessage(msg);
		alert(msg);
	}
}

function migrate(fromVer, toVer) {
	var from = CommonFunc.versionToInt(CommonFunc.versionStrDecode(fromVer));
	var to = CommonFunc.versionToInt(CommonFunc.versionStrDecode(toVer));

	/*var migratePrefs = function(aOldPrefs){
		for(var pref in aOldPrefs) {
			var oldPref = aOldPrefs[pref];
			if(!CommonFunc.prefHasUserValue(oldPref.name)) continue;
			logMessage("Renaming: " + oldPref.name + " -> " + CommonFunc[pref]);
			if(CommonFunc.prefHasUserValue(CommonFunc[pref])) {
				logMessage(CommonFunc[pref] + ": already has user value");
				continue;
			}
			var oldValue = CommonFunc.getPrefValue(oldPref.name, oldPref.type, undefined);
			if(oldValue === undefined) {
				logMessage(oldPref.name + ": cannot get the value");
				continue;
			}
			CommonFunc.setPrefValue(CommonFunc[pref], oldPref.type, oldValue);
			CommonFunc.clearPref(oldPref.name);
		}
	}*/

	/*if(from < 1320) { // migrate from Sage++ 1.3.x
		migratePrefs(CommonFunc.OLD_PREFS);
	}*/

	if(from < 2100) { // migrate from Sage++ 2.0.x
		logMessage("Removing query annotation from root folder");
		_impl.changeRootFolderId(null, _impl.getRootFolderId());
	}

	if(from < 2300) {
		if(CommonFunc.getPrefValue("user_css.enable", "bool", false)) CommonFunc.setPrefValue(CommonFunc.USER_CSS_SELECT, "int", 2);
		["settings.selected_tab", "user_css.enable"].forEach(function(item){ CommonFunc.clearPref(item); });

		var getChromeDir = function(){ return CreateHTML.getSpecialDir("UChrm"); };
		var tmpFile = getChromeDir();
		tmpFile.appendRelativePath("sage.html");
		if(tmpFile.exists()) tmpFile.remove(true);

		tmpFile = getChromeDir();
		tmpFile.append("sagetoo");
		if(tmpFile.exists() && tmpFile.isDirectory()){ // migrate from Sage-Too
			tmpFile.moveTo(null, "sagepp");
			tmpFile.appendRelativePath("sagetoo.html");
			if(tmpFile.exists()) tmpFile.remove(true);
		}
	}
}
//<----- Added by Higmmer : End -----
